Index: src/Fl_File_Icon2.cxx
===================================================================
--- src/Fl_File_Icon2.cxx	(Revision 12532)
+++ src/Fl_File_Icon2.cxx	(Arbeitskopie)
@@ -43,6 +43,7 @@
 #include <FL/math.h>
 #include <FL/Fl_File_Icon.H>
 #include <FL/Fl_Shared_Image.H>
+#include <FL/Fl_Image_Surface.H>
 #include <FL/Fl_Widget.H>
 #include <FL/fl_draw.H>
 #include <FL/filename.H>
@@ -324,6 +325,18 @@
   img = Fl_Shared_Image::get(ifile);
   if (!img || !img->count() || !img->w() || !img->h()) return -1;
 
+#ifdef FLTK_USE_NANOSVG
+  // For .svg images we must ensure they are already rasterized:
+  // With current API this is only possible with an img->draw() call.
+  // In order to avoid X11 warnings (BadDrawable), we wrap this
+  // call into an image surface context.
+  // (No harm for other images types)
+  Fl_Image_Surface *rgb_surf = new Fl_Image_Surface(img->w(), img->h(), 1);
+  Fl_Surface_Device::push_current(rgb_surf);
+  img->draw(0, 0); // rasterizes svg image
+  delete rgb_surf;
+  Fl_Surface_Device::pop_current();
+#endif
   if (img->count() == 1) {
     int		x, y;		// X & Y in image
     int		startx;		// Starting X coord
@@ -663,7 +676,33 @@
 
     snprintf(filename, sizeof(filename), "%s/share/mimelnk", kdedir);
 
-    if (!fl_access(filename, F_OK)) {
+#define UBUNTU_ICON_DIR "/usr/share/icons/Humanity"
+#define UBUNTU (!fl_access(UBUNTU_ICON_DIR, F_OK))
+#ifdef FLTK_USE_NANOSVG
+    // Tested to work on Ubuntu 12.04 to 17.04:
+    // Look for '/usr/share/icons/Humanity' folder *BEFORE* checking kde icons
+    // because the folder '/usr/share/mimelnk' may also be present under
+    // Ubuntu. When found, use the icons in SVG format there.
+    if (UBUNTU) {
+      // Load Ubuntu icons...
+      icon = new Fl_File_Icon("*", Fl_File_Icon::PLAIN);
+      icon->load_image(UBUNTU_ICON_DIR"/mimes/48/text.svg");
+
+      icon = new Fl_File_Icon("*", Fl_File_Icon::DIRECTORY);
+      icon->load_image(UBUNTU_ICON_DIR"/places/48/folder.svg");
+
+      icon = new Fl_File_Icon("*.{bmp|bw|gif|jpg|pbm|pcd|pgm|ppm|png|ras|rgb|tif|xbm|xpm}", Fl_File_Icon::PLAIN);
+      icon->load_image(UBUNTU_ICON_DIR"/mimes/48/image.svg");
+
+      icon = new Fl_File_Icon("*.{svg"
+#ifdef HAVE_LIBZ
+                              "|svgz"
+#endif
+                              "}", Fl_File_Icon::PLAIN);
+      icon->load_image(UBUNTU_ICON_DIR"/mimes/48/svg.svg");
+    } else
+#endif
+    if (!UBUNTU && !fl_access(filename, F_OK)) { // skip for Ubuntu, to (hopefully) use the default icons...
       // Load KDE icons...
       icon = new Fl_File_Icon("*", Fl_File_Icon::PLAIN);
 
